// XML.h: helper class.
//
//////////////////////////////////////////////////////////////////////

#pragma once

#include "../TaskProcessor.h"

#import "msxml.dll"

class CMuleXml
{
public:
	bool Create()
	{
		try
		{
			m_spDoc = NULL;
			m_spDoc.CreateInstance(_T("msxml.domdocument"));
			m_spDoc->async = VARIANT_FALSE;
			return true;
		}
		catch(...)
		{
			return false;
		}
	}

	bool Load(LPCTSTR sXml)
	{
		try
		{
			if(!Create())
				return false;
			return (m_spDoc->loadXML(bstr_t(sXml)) == VARIANT_TRUE);
		}
		catch(...)
		{
			return false;
		}
	}

	bool SelectNode(LPCTSTR sNodeName)
	{
		try
		{
			m_spNode = NULL;
			MSXML::IXMLDOMNode *pResult;
			if(FAILED(m_spDoc->raw_selectSingleNode(bstr_t(sNodeName), &pResult)))
				return false;
			m_spNode = MSXML::IXMLDOMNodePtr(pResult);
			return true;
		}
		catch(...)
		{
			return false;
		}
	}

	CString GetAttribute(LPCTSTR sAttribName)
	{
		try
		{
			MSXML::IXMLDOMNodePtr spName = m_spNode->attributes->getNamedItem(bstr_t(sAttribName));
			if(spName == NULL)
				return _T("");
			bstr_t bsResult = spName->text;
			return (LPCTSTR)bsResult;
		}
		catch(...)
		{
			return _T("");
		}
	}

	bool CreateElement(LPCTSTR sElementName)
	{
		try
		{
/*			MSXML::IXMLDOMProcessingInstructionPtr spPI = 
				m_spDoc->createProcessingInstruction(bstr_t(_T("xml")), bstr_t(_T("version=\"1.0\"")));
			m_spDoc->appendChild(spPI);*/
			m_spNode = m_spDoc->createElement(bstr_t(sElementName));
			if(m_spNode != NULL)
				if(SUCCEEDED(m_spDoc->appendChild(m_spNode)))
					return true;
			return false;
		}
		catch(...)
		{
			return false;
		}
	}

	bool SetAttribute(LPCTSTR sAttribName, variant_t vtValue)
	{
		try
		{
			MSXML::IXMLDOMElementPtr spElem = m_spNode;
			return SUCCEEDED(spElem->setAttribute(bstr_t(sAttribName), vtValue));
		}
		catch(...)
		{
			return false;
		}
	}

	CString GetXml()
	{
		try
		{
			bstr_t bsXml = m_spDoc->xml;
			return (LPCTSTR)bsXml;
		}
		catch(...)
		{
			return _T("");
		}
	}

	MSXML::IXMLDOMNodePtr GetNode(){ return m_spNode; }

public:
	MSXML::IXMLDOMDocumentPtr	m_spDoc;
	MSXML::IXMLDOMNodePtr		m_spNode;
};

class CXmlTask : public CTask, public CMuleXml
{
public:
	CXmlTask();
	virtual bool LoadParams(MSXML::IXMLDOMNodePtr spNode) = 0;
	virtual CString PrepareResult();

public:
	ULONG	m_ulId;
};

// Defining Xml task classes
#define BEGIN_XML_CMD(name)		class CXml_##name : public CXmlTask { public: \
									bool Process(CEmEngine &);\
									bool LoadParams(MSXML::IXMLDOMNodePtr spNode);
#define XML_PARAM_STRING(var)		CString m_str_##var;
#define XML_PARAM_ULONG(var)		ULONG m_ul_##var;
#define END_XML_CMD				};
#include "XMLMsgs.h"

#define XML_PROCESS_FUNC(name)	bool CXml_##name::Process(CEmEngine &stEngine)
