//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


#include "stdafx.h"
#include "emule.h"
#include "packets.h"
#include "sockets.h"
#include "emuleDlg.h"
#include "opcodes.h"
#include "SearchList.h"
#include <time.h>
#include <afxmt.h>
#include <afxinet.h>
#include "UDPSocket.h"
#include "Smtp.h"
#include "HTRichEditCtrl.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#ifdef OLD_SOCKETS_ENABLED

// CServerConnect

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CServerConnect::CServerConnect(CServerList *pServerList, CPreferences *pPrefs)
{
	EMULE_TRY

	m_pConnectedSocket = NULL;
	m_pPrefs = pPrefs;
	m_pUsedServerList = pServerList;
	m_bConnecting = false;
	m_bConnected = false;
	m_dwClientID = 0;
	m_pUDPSocket = new CUDPSocket(this);
	m_pUDPSocket->Create();
	m_iRetryTimerID = 0;
	m_iLastServerListStartPos = 0;
	m_iCheckTimerID = 0;
	m_iNumConnectionAttempts = 0;
	m_pConnectedServerSocket = NULL;
	m_hICCThread = 0;

	InitLocalIP();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CServerConnect::~CServerConnect()
{
	EMULE_TRY

//	Exit ICC Thread if exist
	if (m_hICCThread != 0)
		TerminateThread(m_hICCThread, 0);
//	Stop all connections
	StopConnectionTry();
//	Close connected socket, if any
	DestroySocket(m_pConnectedSocket);
	m_pConnectedSocket = NULL;
//	Close UDP socket
	m_pUDPSocket->Close();
	delete m_pUDPSocket;

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CServerConnect::TryAnotherConnectionRequest()
{
	EMULE_TRY

	if (m_mapConnectionAttempts.GetCount() == 0)
	{
		CServer		*pNextServer = m_pUsedServerList->GetNextServer();

		if (pNextServer == NULL)
		{
			if (m_mapConnectionAttempts.GetCount() == 0)
			{
				AddLogLine(true, IDS_OUTOFSERVERS);
				ConnectToAnyServer(m_iLastServerListStartPos,false);
			}
			return;
		}

	//	Auto-connect to static servers only option
		if (g_eMuleApp.m_pGlobPrefs->AutoConnectStaticOnly())
		{
			if (pNextServer->IsStaticMember())
				ConnectToServer(*pNextServer);
		}
		else
		{
			ConnectToServer(*pNextServer);
		}
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CServerConnect::ConnectToAnyServer(uint32 iStartIndex/*=0xFFFFFFFF*/,bool bPrioritySort/*=true*/,bool bIsAuto/*=true*/)
{
	EMULE_TRY

	CServer		*pNextServer;

//	If no start position is specified start with the next unused server in the list
	if (iStartIndex == 0xFFFFFFFF)
		iStartIndex = m_pUsedServerList->GetServerPosition();
	m_iLastServerListStartPos = iStartIndex;
//	Cancel any ongoing connection attempts
	StopConnectionTry();
//	If we're currently connected to a server, disconnect from it.
	Disconnect();
//	If the server list is empty, output an error message and return
	if (m_pUsedServerList->GetServerCount() == 0)
	{
		AddLogLine(true, RGB_LOG_WARNING + GetResString(IDS_ERR_NOVALIDSERVERSFOUND));
		return;
	}
//	If the "Use priority system" preference is set...
	if (g_eMuleApp.m_pGlobPrefs->Score() && bPrioritySort)
	{
		m_pUsedServerList->Sort();
	}
	m_pUsedServerList->SetServerPosition(iStartIndex);
//	If we're auto-connecting and the "only auto-connect to static server" preference is set...
	if (g_eMuleApp.m_pGlobPrefs->AutoConnectStaticOnly() && bIsAuto)
	{
		bool	bAnyStatic = false;

	//	Scan all servers in the list to find a static one
		for (uint32 i = 0; i < m_pUsedServerList->GetServerCount(); i++)
		{
			pNextServer = m_pUsedServerList->GetNextServer();

		//	If we are at the end of the list, 'pNextServer' will be NULL.
		//	Since the list has at least one server, we just need to repeat operation.
			if (pNextServer == NULL)
				pNextServer = m_pUsedServerList->GetNextServer();

			if (pNextServer->IsStaticMember())
			{
				bAnyStatic = true;
				break;
			}
		}
	//	If we couldn't find a static server, output an error message and return
		if (!bAnyStatic)
		{
			AddLogLine(true, RGB_LOG_WARNING + GetResString(IDS_ERR_NOVALIDSERVERSFOUND));
			return;
		}
	}
//	If we're not auto-connecting or the "only auto-connect to static server" preference is not set...
	else
	{
		pNextServer = m_pUsedServerList->GetNextServer();
	//	If we are at the end of the list, 'pNextServer' will be NULL.
	//	Since the list has at least one server, we just need to repeat operation.
		if (pNextServer == NULL)
			pNextServer = m_pUsedServerList->GetNextServer();
	}
//	Get ready for new upload client connections.
	g_eMuleApp.m_pListenSocket->Process();

//	At this point we have a server object 'pNextServer'. Connect to it.
	if (pNextServer != NULL)
		ConnectToServer(*pNextServer);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CServerConnect::ConnectToServer(CServer &server)
{
	EMULE_TRY

	StopConnectionTry();
	Disconnect();

	m_pConnectedServerSocket = &server;

	CServerSocket		*pNewSocket = new CServerSocket(this);

	if (pNewSocket != NULL)
	{
		m_bConnecting = true;
		m_openSocketList.AddTail(pNewSocket);
		pNewSocket->Create(0,SOCK_STREAM,FD_READ|FD_WRITE|FD_CLOSE|FD_CONNECT,NULL);
		pNewSocket->ConnectToServer(server);

		DWORD		dwCurTick = GetTickCount();

		m_mapConnectionAttempts.SetAt(dwCurTick,pNewSocket);
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	StopConnectionTry() cancels any ongoing server connection attempts and closes all but the currently connected socket.
void CServerConnect::StopConnectionTry()
{
	EMULE_TRY

	m_mapConnectionAttempts.RemoveAll();
	m_bConnecting = false;

	if (m_iRetryTimerID)
	{
		KillTimer(NULL, m_iRetryTimerID);
		m_iRetryTimerID = 0;
	}

//	Close all currently opened sockets except the one which is connected to our current server
	POSITION		pos1, pos2;

	for (pos1 = m_openSocketList.GetHeadPosition(); ( pos2 = pos1 ) != NULL; )
	{
		CServerSocket		*pSck = m_openSocketList.GetNext(pos1);
	//	Don't destroy socket which is connected to server
	//	Don't destroy socket if it is going to destroy itself later on
		if (pSck != m_pConnectedSocket)
			DestroySocket(pSck);
	}
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CServerConnect::ConnectionEstablished(CServerSocket* pServerSocket)
{
	EMULE_TRY

//	Reset the retry code - Don't make any further attempts to connect.
	m_iNumConnectionAttempts = 0;
	if (m_iCheckTimerID)
	  	KillTimer(NULL, m_iCheckTimerID);
	m_iCheckTimerID = 0;

//	We are already connected to another server, so just destroy this socket
	if (m_bConnecting == false)
	{
		if (pServerSocket != m_pConnectedSocket)
			DestroySocket(pServerSocket);
		return;
	}
//	Get the IP of the local computer.
	InitLocalIP();
//	If we're connected and waiting for the server to log us on...
	if (pServerSocket->GetConnectionState() == CS_WAITFORLOGIN)
	{
	//	If there's an entry for the connected socket in the set of connection
	//	attempts, update the attempt time.
		DWORD			dwAttemptTime;
		CServerSocket  *pTempSocket;
		POSITION		pos = m_mapConnectionAttempts.GetStartPosition();

	//	Until we've run out of connecting sockets...
		while (pos)
		{
		//	Retrieve the next key/value entry from the map
			m_mapConnectionAttempts.GetNextAssoc(pos,dwAttemptTime,pTempSocket);
		//	If the entry is for the connected socket...
			if (pTempSocket == pServerSocket)
			{
			//	Remove the entry
				m_mapConnectionAttempts.RemoveKey(dwAttemptTime);
			//	Re-add the entry with the current time
				dwAttemptTime = GetTickCount();
				m_mapConnectionAttempts.SetAt(dwAttemptTime,pServerSocket);
				break;
			}
		}

		AddLogLine( false, IDS_CONNECTEDTOREQ, pServerSocket->m_pServer->GetListName(),
											   pServerSocket->m_pServer->GetFullIP(),
											   pServerSocket->m_pServer->GetPort() );

	//	Send the login packet
		CServer		*pServer = g_eMuleApp.m_pServerList->GetServerByAddress( pServerSocket->m_pServer->GetAddress(),
																			 pServerSocket->m_pServer->GetPort() );
	//	If the server we connected to is in the server list...
		if (pServer)
		{
			pServer->ResetFailedCount();
		//	Update its information in the Server Window.
			g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RefreshServer(*pServer);
		}

	//	Construct the LOGINREQUEST packet byte stream
		CMemFile		packetStream;

		packetStream.Write(g_eMuleApp.m_pGlobPrefs->GetUserHash(),16);	// <HASH> Our userhash

		uint32		dwClientID = GetClientID();							// <ID:DWORD> Our client ID

		packetStream.Write(&dwClientID,sizeof(uint32));

		uint16		nPort = m_pPrefs->GetPort();						// <PORT:WORD> Our TCP port

		packetStream.Write(&nPort,sizeof(uint16));

		uint32		dwTagCount = 4;

		packetStream.Write(&dwTagCount,sizeof(uint32));					// <Tag_set> {<TAGCOUNT:DWORD> 3 tags follow

		CClientTag		tag1(CT_NAME,m_pPrefs->GetUserNick().GetBuffer());

		tag1.WriteToFile(packetStream);									// (CT_NAME:string) Our nick

		CClientTag		tag2(CT_VERSION,EDONKEYVERSION);				// (CT_VERSION:int) Our software version #

		tag2.WriteToFile(packetStream);

		CClientTag		tag3(CT_PORT,m_pPrefs->GetPort());				// (CT_PORT:int) Our port again? }

		tag3.WriteToFile(packetStream);

		CClientTag 		tag4(CT_SUPPORTED_FEATURES,SRV_TCPFLG_COMPRESSION | SRV_TCPFLG_AUXPORT);
		tag4.WriteToFile(packetStream);

		Packet			*pPacket = new Packet(&packetStream);

		pPacket->m_eOpcode = OP_LOGINREQUEST;
		g_eMuleApp.m_pUploadQueue->AddUpDataOverheadServer(pPacket->m_dwSize);
		this->SendPacket(pPacket,true,pServerSocket);
	}
//	If we thought we already were connected...
	else if (pServerSocket->GetConnectionState() == CS_CONNECTED)
	{
		g_eMuleApp.stat_reconnects++;
		g_eMuleApp.stat_serverConnectTime = GetTickCount();
		m_bConnected = true;
		AddLogLine(true, IDS_CONNECTEDTO,pServerSocket->m_pServer->GetListName());
		g_eMuleApp.m_pdlgEmule->ShowConnectionState(true,pServerSocket->m_pServer->GetListName());
		m_pConnectedSocket = pServerSocket;
		StopConnectionTry();
		g_eMuleApp.m_pSharedFilesList->SendListToServer();
		g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RemoveDeadServer();
		if (g_eMuleApp.m_pGlobPrefs->GetAddServersFromConn())
		{
			Packet		*pPacket = new Packet(OP_GETSERVERLIST);

			g_eMuleApp.m_pUploadQueue->AddUpDataOverheadServer(pPacket->m_dwSize);
			SendPacket(pPacket,true);
		}
		if(g_eMuleApp.m_pGlobPrefs->RestartWaiting())
			g_eMuleApp.m_pdlgEmule->m_wndTransfer.m_ctlDownloadList.RestartWaitingDownloads();

		g_eMuleApp.m_pdlgEmule->m_wndServer.m_pctlServerMsgBox->AppendText(CString("\n"));

	//	Reset a request timer after reconnection
		if (pServerSocket->m_pServer != g_eMuleApp.m_pDownloadQueue->GetLastTCPSrcReqServer())
		{
			g_eMuleApp.m_pDownloadQueue->ResetLastTCPRequestTime();
			g_eMuleApp.m_pDownloadQueue->SetLastTCPSrcReqServer(pServerSocket->m_pServer);
		}

	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CServerConnect::SendPacket(Packet *pPacket,bool bDelPacket, CServerSocket *pServerSocket/*=NULL*/)
{
	EMULE_TRY


//	If there's no server socket spec'd...
	if (pServerSocket == NULL)
	{
	//	... and we're connected...
		if (m_bConnected)
		{
			m_pConnectedSocket->SendPacket(pPacket,bDelPacket,true);
			m_pConnectedSocket->m_dwLastTransmission = GetTickCount();
		}
		else
		{
      		if (bDelPacket)
				delete pPacket;

			return false;
		}
	}
	else
	{
		pServerSocket->SendPacket(pPacket,bDelPacket,true);
		pServerSocket->m_dwLastTransmission = GetTickCount();
	}

	return true;

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CServerConnect::SendUDPPacket(Packet *pPacket,CServer *pHostServer,bool bDeletePacket)
{
	EMULE_TRY

	if (m_bConnected && (pHostServer != NULL))
	{
		m_pUDPSocket->SendPacket(pPacket,pHostServer);
	}
	if (bDeletePacket)
		delete pPacket;

	return true;

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CServerConnect::ConnectionFailed(CServerSocket *pServerSocket)
{
	EMULE_TRY

	if (m_bConnecting == false && pServerSocket != m_pConnectedSocket)
	{
	//	Just return, cleanup is done by the socket itself
		return;
	}

	CServer		*pUpdateServer;
	CServer		*pServer = g_eMuleApp.m_pServerList->GetServerByAddress( pServerSocket->m_pServer->GetAddress(),
																		 pServerSocket->m_pServer->GetPort() );

	switch (pServerSocket->GetConnectionState())
	{
		case CS_FATALERROR:
		{
		//	Sending message when internet connection down
			CString		strMessageText = GetResString(IDS_ERR_FATAL);

			g_eMuleApp.m_pSMTPConnection->SendMail( strMessageText,
													g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnServerError(),
													g_eMuleApp.m_pGlobPrefs->IsSMTPWarningEnabled() );
			g_eMuleApp.m_pdlgEmule->ShowNotifier( strMessageText,
												  TBN_SERVER,
												  false,
												  g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnServerError() );
			AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_FATAL));
			m_iNumConnectionAttempts++;
			break;
		}
		case CS_DISCONNECTED:
		{
		//	Sending message when connection lost to server
			CString		strMessageText;

			strMessageText.Format( GetResString(IDS_ERR_LOSTC),
								   pServerSocket->m_pServer->GetListName(),
								   pServerSocket->m_pServer->GetFullIP(),
								   pServerSocket->m_pServer->GetPort() );
			g_eMuleApp.m_pSMTPConnection->SendMail( strMessageText,
													g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnServerError(),
													g_eMuleApp.m_pGlobPrefs->IsSMTPInfoEnabled() );
			g_eMuleApp.m_pdlgEmule->ShowNotifier( strMessageText,
												  TBN_SERVER,
												  false,
												  g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnServerError() );
			g_eMuleApp.m_pdlgEmule->m_wndSharedFiles.m_ctlSharedFilesList.PostMessage(WM_SFL_UPDATEOFFEREDITEMS);
			AddLogLine(true, RGB_LOG_ERROR + GetResString(IDS_ERR_LOSTC),pServerSocket->m_pServer->GetListName(),pServerSocket->m_pServer->GetFullIP(),pServerSocket->m_pServer->GetPort());
			break;
		}
		case CS_SERVERDEAD:
		{
			AddLogLine(false, IDS_ERR_DEAD,pServerSocket->m_pServer->GetListName(),
						pServerSocket->m_pServer->GetFullIP(),pServerSocket->m_pServer->GetPort() );
			pUpdateServer = g_eMuleApp.m_pServerList->GetServerByAddress( pServerSocket->m_pServer->GetAddress(),
																		  pServerSocket->m_pServer->GetPort() );
			if (pUpdateServer)
			{
				pUpdateServer->AddFailedCount();
				g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RefreshServer(*pUpdateServer);
			}
			m_iNumConnectionAttempts++;
			break;
		}
		case CS_ERROR:
		{
			m_iNumConnectionAttempts++;
			break;
		}
		case CS_SERVERFULL:
		{
			AddLogLine(false, IDS_ERR_FULL,pServerSocket->m_pServer->GetListName(),pServerSocket->m_pServer->GetFullIP(),pServerSocket->m_pServer->GetPort());
			break;
		}
		case CS_NOTCONNECTED:
		{
			m_iNumConnectionAttempts++;
			break;
		}
	}

	if (m_iNumConnectionAttempts >= 3)
	{
		if (!m_iCheckTimerID)
		{
			CString		sUrl;

			sUrl.Format(_T("http://%s"), g_eMuleApp.m_ICCURLs[rand() % g_eMuleApp.m_ICCURLs.GetSize()]);

			CString		strMessage = GetResString(IDS_NET_CHECKING) + " [" + sUrl + "]";

			AddLogLine(false, _T("%s"), strMessage);

			if (IsAliveURL(sUrl))
			{
				AddLogLine(false, IDS_NET_WORKING);
				m_iNumConnectionAttempts = 0;
			}
			else if (m_bConnecting)
			{
				AddLogLine(false, RGB_LOG_ERROR + GetResString(IDS_ERR_FATAL));
				StopConnectionTry();
				m_iCheckTimerID = SetTimer(NULL, 0, 1000*CS_RETRYCONNECTTIME, static_cast<TIMERPROC>(CheckInternetCallback));

			//	Refreshing server in list when disconnection occurs
				if (pServer)
					g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RefreshServer(*pServer);

				return;
			}
			else
				return;
		}
	}

	switch (pServerSocket->GetConnectionState())
	{
		case CS_FATALERROR:
		{
			StopConnectionTry();
			if ((m_pPrefs->Reconnect()) && (!m_iRetryTimerID))
			{
   			//	Sending message when reconnecting after 30 seconds
				CString		strMessage;

				strMessage.Format(GetResString(IDS_RECONNECT), CS_RETRYCONNECTTIME);
				g_eMuleApp.m_pSMTPConnection->SendMail( strMessage,
														g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnServerError(),
														g_eMuleApp.m_pGlobPrefs->IsSMTPWarningEnabled() );
				g_eMuleApp.m_pdlgEmule->ShowNotifier( strMessage,
													  TBN_SERVER,
													  false,
													  g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnServerError() );
				AddLogLine(false, IDS_RECONNECT, CS_RETRYCONNECTTIME);
				m_iRetryTimerID = SetTimer(NULL, 0, 1000*CS_RETRYCONNECTTIME, static_cast<TIMERPROC>(RetryConnectCallback));
			}
			break;
		}
		case CS_DISCONNECTED:
		{
			m_bConnected = false;
			if (m_pConnectedSocket)
				m_pConnectedSocket->Close();
			m_pConnectedSocket = NULL;
			g_eMuleApp.m_pdlgEmule->m_dlgSearch.OnBnClickedCancels();

			g_eMuleApp.stat_serverConnectTime = 0;
			g_eMuleApp.m_pdlgEmule->m_dlgStatistics.Add2TotalServerDuration();

			if (m_pPrefs->Reconnect() && !m_bConnecting)
			{
				ConnectToAnyServer();
			}
		//	Sending message when connection lost
			CString		strMessage = GetResString(IDS_CONNECTIONLOST);

			g_eMuleApp.m_pSMTPConnection->SendMail( strMessage,
													g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnServerError(),
													g_eMuleApp.m_pGlobPrefs->IsSMTPWarningEnabled() );
			g_eMuleApp.m_pdlgEmule->ShowNotifier( strMessage,
												  TBN_SERVER,
												  false,
												  g_eMuleApp.m_pGlobPrefs->GetNotifierPopOnServerError() );
			break;
		}
		case CS_ERROR:
		case CS_NOTCONNECTED:
		{
			if (!m_bConnecting)
				break;
			AddLogLine(false, IDS_ERR_CONFAILED,pServerSocket->m_pServer->GetListName(), pServerSocket->m_pServer->GetFullIP(), pServerSocket->m_pServer->GetPort() );
		}
		case CS_SERVERDEAD:
		case CS_SERVERFULL:
		{
			if (!m_bConnecting)
				break;

			DWORD				dwTempKey;
			CServerSocket	   *pTempSocket;
			POSITION			pos = m_mapConnectionAttempts.GetStartPosition();

			while (pos)
			{
				m_mapConnectionAttempts.GetNextAssoc(pos,dwTempKey,pTempSocket);
				if (pTempSocket == pServerSocket)
				{
					m_mapConnectionAttempts.RemoveKey(dwTempKey);
					break;
				}
			}
			TryAnotherConnectionRequest();
		}
	}
//	Refreshing server in list when disconnection occurs
	if (pServer)
		g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RefreshServer(*pServer);

	g_eMuleApp.m_pdlgEmule->ShowConnectionState(false);

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
VOID CALLBACK CServerConnect::RetryConnectCallback(HWND hWnd, UINT nMsg, UINT nId, DWORD dwTime)
{
	EMULE_TRY

//	Always handle all type of MFC exceptions in TimerProcs - otherwise we'll get mem leaks
	CServerConnect		*_this = g_eMuleApp.m_pServerConnect;

	ASSERT(_this);

//	Check if the thread is already running
	if (_this->m_hICCThread && WaitForSingleObject(_this->m_hICCThread, 0) == WAIT_TIMEOUT)
		return;

	_this->StopConnectionTry();

	if (_this->IsConnected())
		return;

	_this->ConnectToAnyServer();

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
VOID CALLBACK CServerConnect::CheckInternetCallback(HWND hWnd, UINT nMsg, UINT nId, DWORD dwTime)
{
	EMULE_TRY

	CServerConnect		*_this= g_eMuleApp.m_pServerConnect;

	ASSERT(_this);

//	Check if the thread is already running
	if (_this->m_hICCThread && WaitForSingleObject(_this->m_hICCThread, 0) == WAIT_TIMEOUT)
		return;

	KillTimer(NULL, _this->m_iCheckTimerID);
	_this->m_iCheckTimerID = 0;

	if (_this->IsConnected())
		return;

	CString		strURL;

	strURL.Format(_T("http://%s"), g_eMuleApp.m_ICCURLs[rand() % g_eMuleApp.m_ICCURLs.GetSize()]);

	CString		strMessage = GetResString(IDS_NET_CHECKING) + " [" + strURL + "]";

	AddLogLine(false, _T("%s"), strMessage);

	if(_this->IsAliveURL(strURL))
	{
		AddLogLine(false, IDS_NET_WORKING);
		_this->m_iNumConnectionAttempts = 0;
		if(!_this->IsConnecting())
			_this->ConnectToAnyServer();
	}
	else
	{
		_this->m_iCheckTimerID = SetTimer(NULL, 0, 1000*CS_RETRYCONNECTTIME, static_cast<TIMERPROC>(CheckInternetCallback));
		if(_this->IsConnecting())
			_this->StopConnectionTry();
	}
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
struct StructIsAliveURL
{
	CString	sURL;
	bool	bSuccess;
};

bool CServerConnect::IsAliveURL(LPCTSTR strURL)
{
	EMULE_TRY

	StructIsAliveURL stData;
	stData.sURL = strURL;
	stData.bSuccess = false;
	m_hICCThread = (HANDLE)_beginthread(IsAliveURLThread, 0, &stData);

	MSG	msg;
	for(bool bDone = false; !bDone; WaitMessage())
	{
		while(PeekMessage(&msg,0,0,0,PM_REMOVE))
		{
			if(msg.message == WM_QUIT)
			{
				bDone = true;
				m_hICCThread = 0;
				PostMessage(NULL,WM_QUIT,0,0);
				break;
			}
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			if(WaitForSingleObject(m_hICCThread, 0) != WAIT_TIMEOUT ||
				!g_eMuleApp.m_pdlgEmule->IsRunning())
			{
				bDone = true;
				m_hICCThread = 0;
				break;
			}
		}
	}

	return stData.bSuccess;

	EMULE_CATCH

	return false;
}

void CServerConnect::IsAliveURLThread(void *pT)
{
	EMULE_TRY

	StructIsAliveURL *stData = (StructIsAliveURL *)pT;
	stData->bSuccess = false;

	HINTERNET		hOpen = InternetOpen(_T(""), INTERNET_OPEN_TYPE_PRECONFIG , NULL, NULL, 0);

	if (!hOpen)
		return;

	HINTERNET		hURL =  InternetOpenUrl(hOpen, stData->sURL, _T(""), NULL, /*INTERNET_FLAG_NO_AUTH | */INTERNET_FLAG_NO_CACHE_WRITE, NULL);

	if (!hURL)
	{
		InternetCloseHandle(hOpen);
		return;
	}

	TCHAR		szBuffer[100];
	DWORD		dwBytes = 100;
	DWORD		dwIndex = 0;
	BOOL		bQuery = HttpQueryInfo(hURL, HTTP_QUERY_STATUS_CODE, szBuffer, &dwBytes, &dwIndex);

	bool		bSuccess = bQuery && !_tcscmp(szBuffer, _T("200"));

	InternetCloseHandle(hURL);
	InternetCloseHandle(hOpen);

	stData->bSuccess = bSuccess;

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CServerConnect::CheckForTimeout()
{
	EMULE_TRY

	DWORD			dwCurTick = GetTickCount();
	DWORD			dwTmpKey;
	CServerSocket  *pTempServerSocket;
	POSITION		pos = m_mapConnectionAttempts.GetStartPosition();

	while (pos)
	{
		m_mapConnectionAttempts.GetNextAssoc(pos,dwTmpKey,pTempServerSocket);

		if (!pTempServerSocket)
		{
			AddDebugLogLine(RGB_LOG_ERROR + _T("%s: Invalid socket during timeout check"), __FUNCTION__);
			m_mapConnectionAttempts.RemoveKey(dwTmpKey);
			return;
		}

		if (dwCurTick - dwTmpKey > CONSERVTIMEOUT)
		{
			AddLogLine( false, IDS_ERR_CONTIMEOUT, pTempServerSocket->m_pServer->GetListName(),
												   pTempServerSocket->m_pServer->GetFullIP(),
												   pTempServerSocket->m_pServer->GetPort() );
			m_mapConnectionAttempts.RemoveKey(dwTmpKey);
			TryAnotherConnectionRequest();
		}
	}
	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CServerConnect::Disconnect()
{
	EMULE_TRY

//	We clear any timer that was left set
	if (m_iCheckTimerID)
	{
	  	KillTimer(NULL, m_iCheckTimerID);
		m_iCheckTimerID = 0;
	}

//	If an ICC Thread is running we terminate it
	if (m_hICCThread != 0)
	{
		if (!TerminateThread(m_hICCThread, 0))
			AddDebugLogLine(RGB_LOG_WARNING + _T("Can't terminate IsAliveURL Thread!"));
	}

//	Socket won't be destroyed if server is changed during connecting
	if ((m_bConnected || m_bConnecting) && m_pConnectedSocket)
	{
		CServer		*pServer = g_eMuleApp.m_pServerList->GetServerByAddress(m_pConnectedSocket->m_pServer->GetAddress(), m_pConnectedSocket->m_pServer->GetPort());

		DestroySocket(m_pConnectedSocket);

		bool		bPreviouslyConnected = m_bConnected;

		m_pConnectedSocket = NULL;
		m_bConnected = false;
	//	Socket won't be destroyed if server is changed during connecting
		m_bConnecting = false;
		g_eMuleApp.m_pdlgEmule->ShowConnectionState(false,_T(""));
		g_eMuleApp.stat_serverConnectTime = 0;
	//	Tell our total server duration to update...
		g_eMuleApp.m_pdlgEmule->m_dlgStatistics.Add2TotalServerDuration();
	//	If previously connected, refresh offered files
		if (bPreviouslyConnected)
			g_eMuleApp.m_pdlgEmule->m_wndSharedFiles.m_ctlSharedFilesList.UpdateOfferedItems();
		if (pServer)
			g_eMuleApp.m_pdlgEmule->m_wndServer.m_ctlServerList.RefreshServer(*pServer);

		return true;
	}

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CServer *CServerConnect::GetCurrentServer()
{
	EMULE_TRY

	if (IsConnected() && m_pConnectedSocket != NULL)
	{
		return m_pConnectedSocket->m_pServer;
	}

	EMULE_CATCH

	return NULL;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
CServer *CServerConnect::GetConnectingServer()
{
	EMULE_TRY

	if (m_bConnecting && m_pConnectedServerSocket != NULL)
		return m_pConnectedServerSocket;

	EMULE_CATCH

	return NULL;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CServerConnect::SetClientID(uint32 newid)
{
	EMULE_TRY

	m_dwClientID = newid;
	g_eMuleApp.m_pdlgEmule->ShowConnectionState(IsConnected(),GetCurrentServer()->GetListName());

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	DestroySocket() closes and destroys socket 'pSck' and removes it from the open socket list.
void CServerConnect::DestroySocket(CServerSocket *pSck)
{
	EMULE_TRY

	if (pSck == NULL)
		return;

//	Remove the socket from the open socket list
	POSITION	posDel = m_openSocketList.Find(pSck);

	if (posDel != NULL)
	{
		m_openSocketList.RemoveAt(posDel);
	}

	if (pSck->m_SocketData.hSocket != INVALID_SOCKET)
	{
	//	Indicate that we don't want notifications for this socket any more.
		pSck->AsyncSelect(0);
	//	Close the socket and release the socket descriptor.
		pSck->Close();
	}

	delete pSck;

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool CServerConnect::IsLocalServer(uint32 dwIP, uint16 nPort)
{
	EMULE_TRY

	if (IsConnected())
	{
		if (m_pConnectedSocket->m_pServer->GetIP() == dwIP && m_pConnectedSocket->m_pServer->GetPort() == nPort)
			return true;
	}

	EMULE_CATCH

	return false;
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	InitLocalIP() gets the IP address of the local computer and stores it in 'm_dwLocalIP'.
void CServerConnect::InitLocalIP()
{
	EMULE_TRY

	m_dwLocalIP = 0;

//	Don't use 'gethostbyname(NULL)'. The winsock DLL may be replaced by a DLL from a third party
//	which is not fully compatible to the original winsock DLL. ppl reported crash with SCORSOCK.DLL
//	when using 'gethostbyname(NULL)'.
	char		szHost[256];

//	Try to get the standard host name for the local computer. If successful...
	if (gethostname(szHost, sizeof szHost) == 0)
	{
		hostent		*pHostEnt = gethostbyname(szHost);

	//	If we received a 4-byte address...
		if (pHostEnt != NULL && pHostEnt->h_length == 4 && pHostEnt->h_addr_list[0] != NULL)
			m_dwLocalIP = *(reinterpret_cast<uint32*>(pHostEnt->h_addr_list[0]));
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CServerConnect::KeepConnectionAlive()
{
	EMULE_TRY

	DWORD dwServerKeepAliveTimeout = g_eMuleApp.m_pGlobPrefs->GetServerKeepAliveTimeout();

	if (dwServerKeepAliveTimeout && m_bConnected && m_pConnectedSocket && m_pConnectedSocket->GetConnectionState() == CS_CONNECTED &&
		GetTickCount() - m_pConnectedSocket->m_dwLastTransmission >= dwServerKeepAliveTimeout)
	{
	//	"Ping" the server if the TCP connection was not used for the specified interval with
	//	an empty publish files packet -> recommended by lugdunummaster himself!
		CSafeMemFile files(4);
		uint32 nFiles = 0;
		files.Write(&nFiles,4);
		Packet *pPacket = new Packet(&files);
		pPacket->m_eOpcode = OP_OFFERFILES;
		g_eMuleApp.m_pUploadQueue->AddUpDataOverheadServer(pPacket->m_dwSize);
		AddDebugLogLine(_T("Refreshing server connection for %s"), m_pConnectedSocket->m_pServer->GetListName());
		m_pConnectedSocket->SendPacket(pPacket,true);
		m_pConnectedSocket->m_dwLastTransmission = GetTickCount();
	}

	EMULE_CATCH
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#endif //OLD_SOCKETS_ENABLED
